Option Strict Off
Option Explicit On
Module CasAPI
	'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	' Declarations of the Windows Smart Card API
	'
	' Note: variable names, such as those in typedef's and function
	'       parameters, have been retained from the C++ API definition
	'
	
	'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	' Defines
	
	' The context is a user context, and any
	' database operations are performed within the
	' domain of the user.
	Public Const SCARD_SCOPE_USER As Integer = 0
	
	' The context is that of the current terminal,
	' and any database operations are performed
	' within the domain of that terminal.  (The
	' calling application must have appropriate
	' access permissions for any database actions.)
	Public Const SCARD_SCOPE_TERMINAL As Integer = 1
	
	' The context is the system context, and any
	' database operations are performed within the
	' domain of the system.  (The calling
	' application must have appropriate access
	' permissions for any database actions.)
	Public Const SCARD_SCOPE_SYSTEM As Integer = 2
	
	' Flag to have Resource Manager allocate buffer space
	Public Const SCARD_AUTOALLOCATE As Integer = -1
	
	' Strings for managing the Smart Card Database
	Public Const SCARD_ALL_READERS As String = "SCard$AllReaders" & vbNullChar & vbNullChar
	Public Const SCARD_DEFAULT_READERS As String = "SCard$DefaultReaders" & vbNullChar & vbNullChar
	Public Const SCARD_LOCAL_READERS As String = "SCard$LocalReaders" & vbNullChar & vbNullChar
	Public Const SCARD_SYSTEM_READERS As String = "SCard$SystemReaders" & vbNullChar & vbNullChar
	
	' Primary Provider Id
	Public Const SCARD_PROVIDER_PRIMARY As Integer = 1
	' Crypto Service Provider Id
	Public Const SCARD_PROVIDER_CSP As Integer = 2
	
	' There is no active protocol
	Public Const SCARD_PROTOCOL_UNDEFINED As Integer = 0
	
	' T=0 is the active protocol
	Public Const SCARD_PROTOCOL_T0 As Integer = 1
	
	' T=1 is the active protocol
	Public Const SCARD_PROTOCOL_T1 As Integer = 2
	
	' Raw is the active protocol.
	Public Const SCARD_PROTOCOL_RAW As Integer = &H10000
	
	' This is the mask of ISO defined transmission protocols
	Public Const SCARD_PROTOCOL_Tx As Integer = SCARD_PROTOCOL_T0 Or SCARD_PROTOCOL_T1
	
	' Use the default transmission parameters / card clock freq.
	Public Const SCARD_PROTOCOL_DEFAULT As Integer = &H80000000
	
	' Memory Card Protocol for Using Castles EZ-Reader
	Public Const SCARD_PROTOCOL_SYNC As Integer = &H10s
	
	
	' Use optimal transmission parameters / card clock freq.
	' Since using the optimal parameters is the default case no bit is defined to be 1
	Public Const SCARD_PROTOCOL_OPTIMAL As Integer = &H0s
	
	' This application is not willing to share this
	' card with other applications.
	Public Const SCARD_SHARE_EXCLUSIVE As Integer = 1
	
	' This application is willing to share this
	' card with other applications.
	Public Const SCARD_SHARE_SHARED As Integer = 2
	
	' This application demands direct control of
	' the reader, so it is not available to other
	' applications.
	Public Const SCARD_SHARE_DIRECT As Integer = 3
	
	' Don't do anything special on close
	Public Const SCARD_LEAVE_CARD As Integer = 0
	
	' Reset the card on close
	Public Const SCARD_RESET_CARD As Integer = 1
	
	' Power down the card on close
	Public Const SCARD_UNPOWER_CARD As Integer = 2
	
	' Eject the card on close
	Public Const SCARD_EJECT_CARD As Integer = 3
	
	' Nothing bigger than this from getAttr
	Public Const MAXIMUM_ATTR_STRING_LENGTH As Integer = 32
	' Limit the readers on the system
	Public Const MAXIMUM_SMARTCARD_READERS As Integer = 10
	
	' This value implies the driver is unaware
	' of the current state of the reader.
	Public Const SCARD_UNKNOWN As Integer = 0
	' This value implies there is no card in
	' the reader.
	Public Const SCARD_ABSENT As Integer = 1
	' This value implies there is a card is
	' present in the reader, but that it has
	' not been moved into position for use.
	Public Const SCARD_PRESENT As Integer = 2
	' This value implies there is a card in the
	'  reader in position for use.  The card is
	'  not powered.
	Public Const SCARD_SWALLOWED As Integer = 3
	' This value implies there is power is
	' being provided to the card, but the
	' Reader Driver is unaware of the mode of
	' the card.
	Public Const SCARD_POWERED As Integer = 4
	' This value implies the card has been
	' reset and is awaiting PTS negotiation.
	Public Const SCARD_NEGOTIABLE As Integer = 5
	' This value implies the card has been
	' reset and specific communication
	' protocols have been established.
	Public Const SCARD_SPECIFIC As Integer = 6
	
	' Ioctl parameters for IOCTL_SMARTCARD_POWER
	' Power down the card.
	Public Const SCARD_POWER_DOWN As Integer = 0
	' Cycle power and reset the card.
	Public Const SCARD_COLD_RESET As Integer = 1
	' Force a reset on the card.
	Public Const SCARD_WARM_RESET As Integer = 2
	
	' Smartcard IOCTL class
	Public Const FILE_DEVICE_SMARTCARD As Integer = &H310000
	
	' Reader action IOCTLs
	Public Const IOCTL_SMARTCARD_POWER As Integer = FILE_DEVICE_SMARTCARD + &H4s
	Public Const IOCTL_SMARTCARD_GET_ATTRIBUTE As Integer = FILE_DEVICE_SMARTCARD + &H8s
	Public Const IOCTL_SMARTCARD_SET_ATTRIBUTE As Integer = FILE_DEVICE_SMARTCARD + &HCs
	Public Const IOCTL_SMARTCARD_CONFISCATE As Integer = FILE_DEVICE_SMARTCARD + &H10s
	Public Const IOCTL_SMARTCARD_TRANSMIT As Integer = FILE_DEVICE_SMARTCARD + &H14s
	Public Const IOCTL_SMARTCARD_EJECT As Integer = FILE_DEVICE_SMARTCARD + &H18s
	Public Const IOCTL_SMARTCARD_SWALLOW As Integer = FILE_DEVICE_SMARTCARD + &H1Cs
	Public Const IOCTL_SMARTCARD_IS_PRESENT As Integer = FILE_DEVICE_SMARTCARD + &H28s
	Public Const IOCTL_SMARTCARD_IS_ABSENT As Integer = FILE_DEVICE_SMARTCARD + &H2Cs
	Public Const IOCTL_SMARTCARD_SET_PROTOCOL As Integer = FILE_DEVICE_SMARTCARD + &H30s
	Public Const IOCTL_SMARTCARD_GET_STATE As Integer = FILE_DEVICE_SMARTCARD + &H38s
	Public Const IOCTL_SMARTCARD_GET_LAST_ERROR As Integer = FILE_DEVICE_SMARTCARD + &H3Cs
	
	' Vendor information definitions - classes
	Public Const SCARD_CLASS_VENDOR_INFO As Integer = &H10000
	' Communication definitions
	Public Const SCARD_CLASS_COMMUNICATIONS As Integer = &H20000
	' Protocol definitions
	Public Const SCARD_CLASS_PROTOCOL As Integer = &H30000
	' Power Management definitions
	Public Const SCARD_CLASS_POWER_MGMT As Integer = &H40000
	' Security Assurance definitions
	Public Const SCARD_CLASS_SECURITY As Integer = &H50000
	' Mechanical characteristic definitions
	Public Const SCARD_CLASS_MECHANICAL As Integer = &H60000
	' Vendor specific definitions
	Public Const SCARD_CLASS_VENDOR_DEFINED As Integer = &H70000
	' Interface Device Protocol options
	Public Const SCARD_CLASS_IFD_PROTOCOL As Integer = &H80000
	' ICC State specific definitions
	Public Const SCARD_CLASS_ICC_STATE As Integer = &H90000
	' System-specific definitions
	Public Const SCARD_CLASS_SYSTEM As Integer = &H7FFF0000
	' Vendor information definitions - items
	Public Const SCARD_ATTR_VENDOR_NAME As Integer = SCARD_CLASS_VENDOR_INFO + &H100s
	Public Const SCARD_ATTR_VENDOR_IFD_TYPE As Integer = SCARD_CLASS_VENDOR_INFO + &H101s
	Public Const SCARD_ATTR_VENDOR_IFD_VERSION As Integer = SCARD_CLASS_VENDOR_INFO + &H102s
	Public Const SCARD_ATTR_VENDOR_IFD_SERIAL_NO As Integer = SCARD_CLASS_VENDOR_INFO + &H103s
	Public Const SCARD_ATTR_CHANNEL_ID As Integer = SCARD_CLASS_COMMUNICATIONS + &H110s
	Public Const SCARD_ATTR_PROTOCOL_TYPES As Integer = SCARD_CLASS_PROTOCOL + &H120s
	Public Const SCARD_ATTR_DEFAULT_CLK As Integer = SCARD_CLASS_PROTOCOL + &H121s
	Public Const SCARD_ATTR_MAX_CLK As Integer = SCARD_CLASS_PROTOCOL + &H122s
	Public Const SCARD_ATTR_DEFAULT_DATA_RATE As Integer = SCARD_CLASS_PROTOCOL + &H123s
	Public Const SCARD_ATTR_MAX_DATA_RATE As Integer = SCARD_CLASS_PROTOCOL + &H124s
	Public Const SCARD_ATTR_MAX_IFSD As Integer = SCARD_CLASS_PROTOCOL + &H125s
	Public Const SCARD_ATTR_POWER_MGMT_SUPPORT As Integer = SCARD_CLASS_POWER_MGMT + &H131s
	Public Const SCARD_ATTR_USER_TO_CARD_AUTH_DEVICE As Integer = SCARD_CLASS_SECURITY + &H140s
	Public Const SCARD_ATTR_USER_AUTH_INPUT_DEVICE As Integer = SCARD_CLASS_SECURITY + &H142s
	Public Const SCARD_ATTR_CHARACTERISTICS As Integer = SCARD_CLASS_MECHANICAL + &H150s
	Public Const SCARD_ATTR_CURRENT_PROTOCOL_TYPE As Integer = SCARD_CLASS_IFD_PROTOCOL + &H201s
	Public Const SCARD_ATTR_CURRENT_CLK As Integer = SCARD_CLASS_IFD_PROTOCOL + &H202s
	Public Const SCARD_ATTR_CURRENT_F As Integer = SCARD_CLASS_IFD_PROTOCOL + &H203s
	Public Const SCARD_ATTR_CURRENT_D As Integer = SCARD_CLASS_IFD_PROTOCOL + &H204s
	Public Const SCARD_ATTR_CURRENT_N As Integer = SCARD_CLASS_IFD_PROTOCOL + &H205s
	Public Const SCARD_ATTR_CURRENT_W As Integer = SCARD_CLASS_IFD_PROTOCOL + &H206s
	Public Const SCARD_ATTR_CURRENT_IFSC As Integer = SCARD_CLASS_IFD_PROTOCOL + &H207s
	Public Const SCARD_ATTR_CURRENT_IFSD As Integer = SCARD_CLASS_IFD_PROTOCOL + &H208s
	Public Const SCARD_ATTR_CURRENT_BWT As Integer = SCARD_CLASS_IFD_PROTOCOL + &H209s
	Public Const SCARD_ATTR_CURRENT_CWT As Integer = SCARD_CLASS_IFD_PROTOCOL + &H20As
	Public Const SCARD_ATTR_CURRENT_EBC_ENCODING As Integer = SCARD_CLASS_IFD_PROTOCOL + &H20Bs
	Public Const SCARD_ATTR_EXTENDED_BWT As Integer = SCARD_CLASS_IFD_PROTOCOL + &H20Cs
	Public Const SCARD_ATTR_ICC_PRESENCE As Integer = SCARD_CLASS_ICC_STATE + &H300s
	Public Const SCARD_ATTR_ICC_INTERFACE_STATUS As Integer = SCARD_CLASS_ICC_STATE + &H301s
	Public Const SCARD_ATTR_CURRENT_IO_STATE As Integer = SCARD_CLASS_ICC_STATE + &H302s
	Public Const SCARD_ATTR_ATR_STRING As Integer = SCARD_CLASS_ICC_STATE + &H303s
	Public Const SCARD_ATTR_ICC_TYPE_PER_ATR As Integer = SCARD_CLASS_ICC_STATE + &H304s
	Public Const SCARD_ATTR_ESC_RESET As Integer = SCARD_CLASS_VENDOR_DEFINED + &HA000s
	Public Const SCARD_ATTR_ESC_CANCEL As Integer = SCARD_CLASS_VENDOR_DEFINED + &HA003s
	Public Const SCARD_ATTR_ESC_AUTHREQUEST As Integer = SCARD_CLASS_VENDOR_DEFINED + &HA005s
	Public Const SCARD_ATTR_MAXINPUT As Integer = SCARD_CLASS_VENDOR_DEFINED + &HA007s
	Public Const SCARD_ATTR_DEVICE_UNIT As Integer = SCARD_CLASS_SYSTEM + &H1s
	Public Const SCARD_ATTR_DEVICE_IN_USE As Integer = SCARD_CLASS_SYSTEM + &H2s
	Public Const SCARD_ATTR_DEVICE_FRIENDLY_NAME_A As Integer = SCARD_CLASS_SYSTEM + &H3s
	Public Const SCARD_ATTR_DEVICE_SYSTEM_NAME_A As Integer = SCARD_CLASS_SYSTEM + &H4s
	Public Const SCARD_ATTR_DEVICE_FRIENDLY_NAME_W As Integer = SCARD_CLASS_SYSTEM + &H5s
	Public Const SCARD_ATTR_DEVICE_SYSTEM_NAME_W As Integer = SCARD_CLASS_SYSTEM + &H6s
	Public Const SCARD_ATTR_SUPRESS_T1_IFS_REQUEST As Integer = SCARD_CLASS_SYSTEM + &H7s
	
	' types for tracking cards within readers
	
	' state of a reader
	Structure SCARD_READERSTATEA
		' reader name
		Dim szReader As String
		' user defined data
		Dim pvUserData As Integer
		' current state of reader at time of call
		Dim dwCurrentState As Integer
		' state of reader after state change
		Dim dwEventState As Integer
		' Number of bytes in the returned ATR
		Dim cbAtr As Integer
		' Atr of inserted card, (extra alignment bytes)
		<VBFixedArray(35)> Dim rgbAtr() As Byte
		
		'UPGRADE_TODO: Is "Initialize" HlƦcC @UHoԲӸT: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1026"'
		Public Sub Initialize()
			ReDim rgbAtr(35)
		End Sub
	End Structure
	
	' The application is unaware of the current state, and would like to
	' know.  The use of this value results in an immediate return
	' from state transition monitoring services.  This is represented by
	' all bits set to zero.
	Public Const SCARD_STATE_UNAWARE As Integer = &H0s
	
	' The application requested that this reader be ignored.  No other
	' bits will be set.
	Public Const SCARD_STATE_IGNORE As Integer = &H1s
	
	' This implies that there is a difference between the state
	' believed by the application, and the state known by the Service
	' Manager.  When this bit is set, the application may assume a
	' significant state change has occurred on this reader.
	Public Const SCARD_STATE_CHANGED As Integer = &H2s
	
	' This implies that the given reader name is not recognized by
	' the Service Manager.  If this bit is set, then SCARD_STATE_CHANGED
	' and SCARD_STATE_IGNORE will also be set.
	Public Const SCARD_STATE_UNKNOWN As Integer = &H4s
	
	' This implies that the actual state of this reader is not
	' available.  If this bit is set, then all the following bits are clear.
	Public Const SCARD_STATE_UNAVAILABLE As Integer = &H8s
	
	' This implies that there is not card in the reader.  If this bit
	' is set, all the following bits will be clear.
	Public Const SCARD_STATE_EMPTY As Integer = &H10s
	
	' This implies that there is a card in the reader.
	Public Const SCARD_STATE_PRESENT As Integer = &H20s
	
	' This implies that there is a card in the reader with an ATR
	' matching one of the target cards.  If this bit is set,
	' SCARD_STATE_PRESENT will also be set.  This bit is only returned
	' on the SCardLocateCard() service.
	Public Const SCARD_STATE_ATRMATCH As Integer = &H40s
	
	' This implies that the card in the reader is allocated for exclusive
	' use by another application.  If this bit is set,
	' SCARD_STATE_PRESENT will also be set.
	Public Const SCARD_STATE_EXCLUSIVE As Integer = &H80s
	
	' This implies that the card in the reader is in use by one or more'
	' other applications, but may be connected to in shared mode.  If
	' this bit is set, SCARD_STATE_PRESENT will also be set.
	Public Const SCARD_STATE_INUSE As Integer = &H100s
	
	' This implies that the card in the reader is unresponsive or not
	' supported by the reader or software.
	Public Const SCARD_STATE_MUTE As Integer = &H200s
	
	' This implies that the card in the reader has not been powered up.
	Public Const SCARD_STATE_UNPOWERED As Integer = &H400s
	
	' types for providing access to the I/O capabilities of the reader drivers
	
	' I/O request control
	Structure SCARD_IO_REQUEST
		' Protocol identifier
		Dim dwProtocol As Integer
		' Protocol Control Information Length
		Dim dbPciLength As Integer
	End Structure
	
	' T=0 command
	Structure SCARD_T0_Command
		' the instruction class
		Dim bCla As Byte
		' the instruction code within the instruction class
		Dim bIns As Byte
		' first parameter of the function
		Dim bP1 As Byte
		' second parameter of the function
		Dim bP2 As Byte
		' size of the I/O transfer
		Dim bP3 As Byte
	End Structure
	
	' T=0 request
	Structure SCARD_T0_REQUEST
		' I/O request control
		Dim ioRequest As SCARD_IO_REQUEST
		' first return code from the instruction
		Dim bSw1 As Byte
		' second return code from the instruction
		Dim bSw2 As Byte
		' I/O command
		Dim CmdBytes As SCARD_T0_Command
	End Structure
	
	' T=1 request
	Structure SCARD_T1_REQUEST
		' I/O request control
		Dim ioRequest As SCARD_IO_REQUEST
	End Structure
	
	' smart card dialog definitions
	
	' show UI only if required to select card
	Public Const SC_DLG_MINIMAL_UI As Integer = 1
	
	' do not show UI in any case
	Public Const SC_DLG_NO_UI As Integer = 2
	
	' show UI in every case
	Public Const SC_DLG_FORCE_UI As Integer = 4
	
	' dialog error returns
	Public Const SCERR_NOCARDNAME As Integer = &H4000s
	Public Const SCERR_NOGUIDS As Integer = &H8000s
	
	
	'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	' Functions
	
	'UPGRADE_ISSUE: 䴩ŧiѼ 'As Any'C @UHoԲӸT: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1016"'
    Public Declare Function CasEstablishContext Lib "casmcard.dll" (ByVal dwScope As Integer, ByVal pvReserved1 As Integer, ByVal pvReserved2 As Integer, ByRef phContext As Integer) As Integer
	
	Public Declare Function CasReleaseContext Lib "casmcard.dll" (ByVal hContext As Integer) As Integer
	
	Public Declare Function CasFreeMemory Lib "casmcard.dll" (ByVal hContext As Integer, ByVal pvMem As Integer) As Integer
	
	Public Declare Function CasCancel Lib "casmcard.dll" (ByVal hContext As Integer) As Integer
	
	Public Declare Function CasReconnect Lib "casmcard.dll" (ByVal hCard As Integer, ByVal dwShareMode As Integer, ByVal dwPreferredProtocols As Integer, ByVal dwInitialization As Integer, ByRef pdwActiveProtocol As Object) As Integer
	
	Public Declare Function CasDisconnect Lib "casmcard.dll" (ByVal hCard As Integer, ByVal dwDisposition As Integer) As Integer
	
	Public Declare Function CasBeginTransaction Lib "casmcard.dll" (ByVal hCard As Integer) As Integer
	
	Public Declare Function CasEndTransaction Lib "casmcard.dll" (ByVal hCard As Integer, ByVal dwDisposition As Integer) As Integer
	
	'UPGRADE_WARNING: c SCARD_IO_REQUEST ݭnNʰeBzݩʦb Declare z@޼ƶǻC @UHoԲӸT: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1050"'
	Public Declare Function CasTransmit Lib "casmcard.dll" (ByVal hCard As Integer, ByRef pioSendPci As SCARD_IO_REQUEST, ByRef pbSendBuffer As Byte, ByVal cbSendLength As Integer, ByRef pioRecvPci As SCARD_IO_REQUEST, ByRef pbRecvBuffer As Byte, ByRef pcbRecvLength As Integer) As Integer
	
	Public Declare Function CasControl Lib "casmcard.dll" (ByVal hCard As Integer, ByVal dwControlCode As Integer, ByRef pvInBuffer As Byte, ByVal cbInBufferSize As Integer, ByRef pvOutBuffer As Byte, ByVal cbOutBufferSize As Integer, ByRef pcbBytesReturned As Integer) As Integer
	
	Public Declare Function CasGetAttrib Lib "casmcard.dll" (ByVal hCard As Integer, ByVal dwAttrId As Integer, ByRef pbAttr As Byte, ByRef pcbAttrLen As Integer) As Integer
	
	Public Declare Function CasSetAttrib Lib "casmcard.dll" (ByVal hCard As Integer, ByVal dwAttrId As Integer, ByRef pbAttr As Byte, ByVal cbAttrLen As Integer) As Integer
	
	Public Declare Function CasListReaderGroupsA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal mszGroups As String, ByRef pcchGroups As Integer) As Integer
	
	Public Declare Function CasListReadersA Lib "casmcard.dll" (ByVal SCARDCONTEXT As Integer, ByVal mszGroups As String, ByVal mszReaders As String, ByRef pcchReaders As Integer) As Integer
	
	'UPGRADE_WARNING: c GUID ݭnNʰeBzݩʦb Declare z@޼ƶǻC @UHoԲӸT: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1050"'
	Public Declare Function CasListCardsA Lib "casmcard.dll" (ByVal hContext As Integer, ByRef pbAtr As Byte, ByRef rgguidInterfaces As GUID, ByVal cguidInterfaceCount As Integer, ByVal mszCards As String, ByRef pcchCards As Integer) As Integer
	
	' second declaration for passing NULL through ATR and GUID parameters
	Public Declare Function CasListCardsA2 Lib "casmcard.dll"  Alias "CasListCardsA"(ByVal hContext As Integer, ByVal pbAtr As Integer, ByVal rgguidInterfaces As Integer, ByVal cguidInterfaceCount As Integer, ByVal mszCards As String, ByRef pcchCards As Integer) As Integer
	
	' third declaration for passing NULL through ATR and GUID parameters, BYTE name parameter
	Public Declare Function CasListCardsA3 Lib "casmcard.dll"  Alias "CasListCardsA"(ByVal hContext As Integer, ByVal pbAtr As Integer, ByVal rgguidInterfaces As Integer, ByVal cguidInterfaceCount As Integer, ByRef mszCards As Byte, ByRef pcchCards As Integer) As Integer
	
	'UPGRADE_WARNING: c GUID ݭnNʰeBzݩʦb Declare z@޼ƶǻC @UHoԲӸT: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1050"'
	Public Declare Function CasListInterfacesA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal szCard As String, ByRef pguidInterfaces As GUID, ByRef pcguidInterfaces As Integer) As Integer
	
	'UPGRADE_WARNING: c GUID ݭnNʰeBzݩʦb Declare z@޼ƶǻC @UHoԲӸT: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1050"'
	Public Declare Function CasGetProviderIdA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal szCard As String, ByRef pguidProviderId As GUID) As Integer
	
	Public Declare Function CasGetCardTypeProviderNameA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal szCardName As String, ByVal dwProviderId As Integer, ByVal szProvider As String, ByRef pcchProvider As Integer) As Integer
	
	Public Declare Function CasIntroduceReaderGroupA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal szGroupName As String) As Integer
	
	Public Declare Function CasForgetReaderGroupA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal szGroupName As String) As Integer
	
	Public Declare Function CasIntroduceReaderA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal szReadeName As String, ByVal szDeviceName As String) As Integer
	
	Public Declare Function CasForgetReaderA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal szReaderName As String) As Integer
	
	Public Declare Function CasAddReaderToGroupA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal szReaderName As String, ByVal szGroupName As String) As Integer
	
	Public Declare Function CasRemoveReaderFromGroupA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal szReaderName As String, ByVal szGroupName As String) As Integer
	
	'UPGRADE_WARNING: c GUID ݭnNʰeBzݩʦb Declare z@޼ƶǻC @UHoԲӸT: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1050"'
	Public Declare Function CasIntroduceCardTypeA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal szCardName As String, ByRef pguidPrimaryProvider As GUID, ByRef pguidInterfaces As GUID, ByVal dwInterfaceCount As Integer, ByVal pbAtr As String, ByVal pbAtrMask As String, ByVal cbAtrLen As Integer) As Integer
	
	Public Declare Function CasSetCardTypeProviderNameA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal szCardName As String, ByVal dwProviderId As Integer, ByVal szProvider As String) As Integer
	
	Public Declare Function CasForgetCardTypeA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal szCardName As String) As Integer
	
	'UPGRADE_WARNING: c SCARD_READERSTATEA ݭnNʰeBzݩʦb Declare z@޼ƶǻC @UHoԲӸT: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1050"'
	Public Declare Function CasLocateCardsA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal mszCards As String, ByRef rgReaderStates As SCARD_READERSTATEA, ByVal cReaders As Integer) As Integer
	
	'UPGRADE_WARNING: c SCARD_READERSTATEA ݭnNʰeBzݩʦb Declare z@޼ƶǻC @UHoԲӸT: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1050"'
	Public Declare Function CasGetStatusChangeA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal dwTimeout As Integer, ByRef rgReaderStates As SCARD_READERSTATEA, ByVal cReaders As Integer) As Integer
	
	Public Declare Function CasConnectA Lib "casmcard.dll" (ByVal hContext As Integer, ByVal szReader As String, ByVal dwShareMode As Integer, ByVal dwPreferredProtocols As Integer, ByRef phCard As Integer, ByRef pdwActiveProtocol As Integer) As Integer
	
	Public Declare Function CasStatusA Lib "casmcard.dll" (ByVal hCard As Integer, ByVal mszReaderNames As String, ByRef pcchReaderLen As Integer, ByRef pdwState As Integer, ByRef pdwProtocol As Integer, ByRef pbAtr As Byte, ByRef pcbAtrLen As Integer) As Integer
	
	
	'====================================================================================
	'
	'  Casltes Memory Card API
	'
	'====================================================================================
	
	' SLE4432
	Public Declare Function SLE4432_Read_Main_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef RecvBuffer As Byte, ByVal StartAddr As Integer, ByRef RecvBufferLength As Integer) As Integer
	
	
	
	Public Declare Function SLE4432_Update_Main_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4432_Update_Main_MemoryA Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4432_Read_Protection_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef RecvBuffer As Byte) As Integer
	
	Public Declare Function SLE4432_Write_Protection_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	'====================================================================================
	' SLE4442
	Public Declare Function SLE4442_Read_Main_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef RecvBuffer As Byte, ByVal StartAddr As Integer, ByRef RecvBufferLength As Integer) As Integer
	
	
	
	Public Declare Function SLE4442_Update_Main_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4442_Update_Main_MemoryA Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4442_Read_Protection_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef RecvBuffer As Byte) As Integer
	
	Public Declare Function SLE4442_Write_Protection_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	
	Public Declare Function SLE4442_Read_Security_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef RecvBuffer As Byte) As Integer
	
	Public Declare Function SLE4442_Update_Security_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4442_Compare_Verification_Data Lib "casmcard.dll" (ByVal hCard As Integer, ByVal PSC1 As Byte, ByVal PSC2 As Byte, ByVal PSC3 As Byte) As Integer
	
	'====================================================================================
	'SLE4418
	
	Public Declare Function SLE4418_Write_Data_With_Protect_Bit Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4418_Write_Data_With_Protect_BitA Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4418_Write_Data_Without_Protect_Bit Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4418_Write_Data_Without_Protect_BitA Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4418_Write_Protect_Bit Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4418_Read_Data_With_Protect_Bit Lib "casmcard.dll" (ByVal hCard As Integer, ByRef RecvBuffer As Byte, ByRef ProtectBuffer As Byte, ByVal StartAddr As Integer, ByRef RecvBufferLength As Integer) As Integer
	
	Public Declare Function SLE4418_Read_Data_Without_Protect_Bit Lib "casmcard.dll" (ByVal hCard As Integer, ByRef RecvBuffer As Byte, ByVal StartAddr As Integer, ByRef RecvBufferLength As Integer) As Integer
	
	
	'====================================================================================
	'SLE4428
	
	Public Declare Function SLE4428_Write_Data_With_Protect_Bit Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4428_Write_Data_With_Protect_BitA Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4428_Write_Data_Without_Protect_Bit Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4428_Write_Data_Without_Protect_BitA Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4428_Write_Protect_Bit Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4428_Read_Data_With_Protect_Bit Lib "casmcard.dll" (ByVal hCard As Integer, ByRef RecvBuffer As Byte, ByRef ProtectBuffer As Byte, ByVal StartAddr As Integer, ByRef RecvBufferLength As Integer) As Integer
	
	Public Declare Function SLE4428_Read_Data_Without_Protect_Bit Lib "casmcard.dll" (ByVal hCard As Integer, ByRef RecvBuffer As Byte, ByVal StartAddr As Integer, ByRef RecvBufferLength As Integer) As Integer
	
	Public Declare Function SLE4428_PSC_Verification Lib "casmcard.dll" (ByVal hCard As Integer, ByVal PSC1 As Byte, ByVal PSC2 As Byte) As Integer
	
	
	
	'====================================================================================
	'SLE4404
	
	Public Declare Function SLE4404_Read_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef RecvBuffer As Byte, ByVal StartAddr As Integer, ByRef RecvBufferLength As Integer) As Integer
	
	Public Declare Function SLE4404_Read_User_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef RecvBuffer As Byte, ByVal StartAddr As Integer, ByRef RecvBufferLength As Integer) As Integer
	
	Public Declare Function SLE4404_Compare_User_Code Lib "casmcard.dll" (ByVal hCard As Integer, ByVal Code1 As Byte, ByVal Code2 As Byte) As Integer
	
	Public Declare Function SLE4404_Compare_Memory_Code Lib "casmcard.dll" (ByVal hCard As Integer, ByVal Code1 As Byte, ByVal Code2 As Byte, ByVal Code3 As Byte, ByVal Code4 As Byte) As Integer
	
	Public Declare Function SLE4404_Write_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	
	Public Declare Function SLE4404_Erase_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByVal StartAddr As Integer, ByVal EraseLength As Integer) As Integer
	
	Public Declare Function SLE4404_Write_MemoryA Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4404_Write_User_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4404_Write_User_MemoryA Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4404_Enter_Test_Mode Lib "casmcard.dll" (ByVal hCard As Integer) As Integer
	
	Public Declare Function SLE4404_Exit_Test_Mode Lib "casmcard.dll" (ByVal hCard As Integer) As Integer
	
	Public Declare Function SLE4404_Blow_Fuse Lib "casmcard.dll" (ByVal hCard As Integer) As Integer
	
	'====================================================================================
	'SLE4406
	
	Public Declare Function SLE4406_Read_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef RecvBuffer As Byte, ByVal StartAddr As Integer, ByRef RecvBufferLength As Integer) As Integer
	
	Public Declare Function SLE4406_Write_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4406_Read_Counter_Stages Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte) As Integer
	
	Public Declare Function SLE4406_Write_Counter_Stage Lib "casmcard.dll" (ByVal hCard As Integer, ByVal Stage As Byte, ByVal Data As Byte) As Integer
	
	
	Public Declare Function SLE4406_Reload Lib "casmcard.dll" (ByVal hCard As Integer, ByVal BitAddr As Byte) As Integer
	
	
	'====================================================================================
	'SLE4436
	
	
	Public Declare Function SLE4436_Read_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef RecvBuffer As Byte, ByVal StartAddr As Integer, ByRef RecvBufferLength As Integer) As Integer
	
	Public Declare Function SLE4436_Write_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE4436_Read_Counter_Stages Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte) As Integer
	
	Public Declare Function SLE4436_Write_Counter_Stage Lib "casmcard.dll" (ByVal hCard As Integer, ByVal Stage As Byte, ByVal Data As Byte) As Integer
	
	
	Public Declare Function SLE4436_Reload Lib "casmcard.dll" (ByVal hCard As Integer, ByVal BitAddr As Byte) As Integer
	
	Public Declare Function SLE4436_Verify_Transport_Code Lib "casmcard.dll" (ByVal hCard As Integer, ByVal Code1 As Byte, ByVal Code2 As Byte, ByVal Code3 As Byte) As Integer
	
	Public Declare Function SLE4436_Authentication Lib "casmcard.dll" (ByVal hCard As Integer, ByRef ReturnCode As Byte, ByVal Key As Byte, ByVal ClockPulse As Byte, ByRef Code As Byte) As Integer
	
	
	'====================================================================================
	'SLE5536
	
	
	Public Declare Function SLE5536_Read_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef RecvBuffer As Byte, ByVal StartAddr As Integer, ByRef RecvBufferLength As Integer) As Integer
	
	Public Declare Function SLE5536_Write_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte, ByVal StartAddr As Integer, ByVal SendBufferLength As Integer) As Integer
	
	Public Declare Function SLE5536_Read_Counter_Stages Lib "casmcard.dll" (ByVal hCard As Integer, ByRef SendBuffer As Byte) As Integer
	
	Public Declare Function SLE5536_Write_Counter_Stage Lib "casmcard.dll" (ByVal hCard As Integer, ByVal Stage As Byte, ByVal Data As Byte) As Integer
	
	
	Public Declare Function SLE5536_Reload Lib "casmcard.dll" (ByVal hCard As Integer, ByVal BitAddr As Byte) As Integer
	
	Public Declare Function SLE5536_Verify_Transport_Code Lib "casmcard.dll" (ByVal hCard As Integer, ByVal Code1 As Byte, ByVal Code2 As Byte, ByVal Code3 As Byte) As Integer
	
	Public Declare Function SLE5536_Authentication Lib "casmcard.dll" (ByVal hCard As Integer, ByRef ReturnCode As Byte, ByVal Key As Byte, ByVal ClockPulse As Byte, ByRef Code As Byte) As Integer
	
	
	'====================================================================================
	'ICC
	
	Public Declare Function I2C_Read_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByVal ReadCommand As Byte, ByVal StartAddr As Integer, ByRef RecvBuffer As Byte, ByRef RecvBufferLength As Integer, ByVal Dummy As Byte, ByVal AddrByteNum As Byte) As Integer
	
	
	Public Declare Function I2C_Write_Memory Lib "casmcard.dll" (ByVal hCard As Integer, ByVal WriteCommand As Byte, ByVal StartAddr As Integer, ByRef SendBuffer As Byte, ByVal SendBufferLength As Integer, ByVal AddrByteNum As Byte) As Integer
	
	Public Declare Function I2C_SetMode Lib "casmcard.dll" (ByVal hCard As Integer) As Integer
	
    Public Declare Function I2C_ClearMode Lib "casmcard.dll" (ByVal hCard As Integer) As Integer
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' Helper Defines

    ' Null value suitable for passing in as parameter
    Public Const lngNull As Long = 0

    ' GUID data type
    ' flags for API error message reporting
    Enum EFORMAT_MESSAGE
        FORMAT_MESSAGE_NONE = &H0
        FORMAT_MESSAGE_ALLOCATE_BUFFER = &H100
        FORMAT_MESSAGE_IGNORE_INSERTS = &H200
        FORMAT_MESSAGE_FROM_STRING = &H400
        FORMAT_MESSAGE_FROM_HMODULE = &H800
        FORMAT_MESSAGE_FROM_SYSTEM = &H1000
        FORMAT_MESSAGE_ARGUMENT_ARRAY = &H2000
        FORMAT_MESSAGE_MAX_WIDTH_MASK = &HFF
    End Enum


    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' Helper Functions

    ' pull an error message from a system image
    Public Declare Function FormatMessageA _
    Lib "kernel32" _
    ( _
        ByVal dwFlags As Long, _
        ByVal lpSources As Long, _
        ByVal dwMessageId As Long, _
        ByVal dwLanguageId As Long, _
        ByVal lpBuffer As String, _
        ByVal nSize As Long, _
        ByVal Arguments As Long _
        ) As Long


    ' ParseMultistring
    '
    ' converts C++ type multistring to VB type array of strings
    ' and reports the number of strings in array
    '
    ' assumptions:
    '
    '   first parameter actually contains a good format multistring
    '   no checking is done for bad format, empty string, or null pointer
    '
    ' arguments:
    '
    '   strMultistring - supplies the multistring to be converted,
    '       whose format is concatenated null terminated strings, with an
    '       additional null terminator at the end of the last string
    '
    '   intReaderCount - returns the number of strings in the array
    '       note that an array with 4 strings will have indexes 0 to 3
    '
    ' return value:
    '
    '   an array of strings - can contain one empty element if no readers found
    '   indirectly through second parameter, the count of strings in the array
    '
    Public Function ParseMultistring( _
        ByRef strMultistring As String, _
        ByRef intReaderCount As Integer _
        ) As Object

        ' string parsed out from multistring
        Dim strCurrent As String
        ' copy of multistring, so that calling program still has a good copy
        Dim strWorking As String
        ' reader names in array of strings
        Dim arrReaderNames() As String
        ' position of first null terminator within the multistring
        Dim lngNullPosition As Long

        ' initialize current string to empty
        strCurrent = ""
        ' make a copy of the input multistring
        strWorking = strMultistring
        ' set count of readers to zero
        intReaderCount = 0
        ' get position of first null terminator
        lngNullPosition = InStr(strWorking, vbNullChar)

        ' when all strings and their individual null terminators have been
        ' parsed out of the multistring, only the final null terminator will remain
        While (lngNullPosition > 1)
            ' resize the array to add another element, preserving old elements
            ReDim Preserve arrReaderNames(intReaderCount)
            ' parse out the first string in the multistring
            strCurrent = Left(strWorking, lngNullPosition - 1)
            ' copy this string into the array
            arrReaderNames(intReaderCount) = strCurrent
            ' delete this string from the multistring
            strWorking = Right(strWorking, Len(strWorking) - (lngNullPosition + 1) + 1)
            ' get position of the first null terminator
            lngNullPosition = InStr(strWorking, vbNullChar)
            ' increase the string count by one for the string just parsed
            intReaderCount = intReaderCount + 1
        End While

        ' return the completed array
        ParseMultistring = arrReaderNames

    End Function ' ParseMultistring


End Module